@@ -18,7 +18,7 @@
 */
 
 static char *electricsheep_c_id =
-"@(#) $Id: electricsheep.c,v 1.56 2005/12/13 06:55:28 spotspot Exp $";
+"@(#) $Id: electricsheep.c,v 1.59 2006/04/14 08:59:39 spotspot Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -100,9 +100,6 @@ char *logfile = NULL;
 time_t bracket_begin_time = (time_t)(-1);
 time_t bracket_end_time = (time_t)(-1);
 
-
-int nplays_by_id[max_anims];
-
 int nrepeats = 2;
 int nthreads = -1;
 
@@ -110,7 +107,7 @@ char *dream_server = "v2d6.sheepserver.net";
 
 // cheeze XXX
 #define max_cp_size 300000
-#define MAXBUF (5*MAXPATHLEN)
+#define MAXBUF (5*PATH_MAX)
 
 char *play_prog = "mpeg2dec_onroot";
 char *nick_name = "";
@@ -183,15 +180,14 @@ typedef struct {
     int rating;
     int first;
     int last;
+    char path[PATH_MAX];
     char url[max_url_length];
 } anim_t;
 
 
-int nserver_anims;
-int ncached_anims;
-
-anim_t server_anims[max_anims];
-anim_t cached_anims[max_anims];
+int nserver_anims, ncached_anims;
+anim_t *server_anims = NULL;
+anim_t *cached_anims = NULL;
 
 char *thread_name = NULL;
 
@@ -317,11 +313,11 @@ print_anims(char *name, anim_t *an, int nanims) {
     printf("%s=%d\n", name, nanims);
     for (i = 0; i < nanims; i++) {
 	printf("gen=%d id=%d deleted=%d type=%d ctime=%d size=%d rating=%d "
-	       "first=%d last=%d url=%s\n",
+	       "first=%d last=%d path=%s url=%s\n",
 	       an[i].generation,
 	       an[i].id, an[i].deleted, an[i].type, an[i].ctime,
 	       an[i].size, an[i].rating, an[i].first, an[i].last,
-	       an[i].url);
+	       an[i].path, an[i].url);
     }
 }
 
@@ -337,26 +333,66 @@ filename_is_xxx(char *name) {
     return !(n <= 4 || 0 != strcmp(&name[n-4], ".xxx"));
 }
 
-void
-update_cached_anims(int match_gen) {
+void clear_path(char *path, int generation) {
+    DIR *d;
+    struct dirent *e;
+    char fbuf[PATH_MAX];
+    struct stat sbuf;
+    
+    if (debug) printf("killing generation %d in %s!\n", generation, path);
+
+    d = opendir(path);
+
+    if (!d) {
+	perror(path);
+	cleanup_and_exit(1);
+    }
+
+    while (e = readdir(d)) {
+	int g;
+	snprintf(fbuf, PATH_MAX, "%s%s", path, e->d_name);
+	
+	if (e->d_name[0] == '.') continue;
+	if (0 == strncmp("gen", e->d_name, 3)) {
+	    unlink(fbuf);
+	    continue;
+	}
+	if (filename_is_mpg(e->d_name) || filename_is_xxx(e->d_name)) {
+	    if (1 == sscanf(e->d_name, "%d=", &g) && generation == g) {
+		unlink(fbuf);
+		continue;
+	    }
+	}
+	if (-1 == stat(fbuf, &sbuf)) continue;
+	if (S_ISDIR(sbuf.st_mode)) {
+	    strncat(fbuf, "/", PATH_MAX);
+	    clear_path(fbuf, generation);
+	}
+    }
+    closedir(d);
+}
+
+void update_cached_anims_path(char *path, int match_gen) {
     int write_gen = 0;
     int checked_gen = 0;
     int clear_gen = 0;
-    DIR *d = opendir(leave_prefix);
+    DIR *d;
     struct dirent *e;
     struct stat sbuf;
     char fbuf[MAXBUF];
+    int old_gen = -1;
 
+    if (debug) printf("updating cache path=%s match_gen=%d\n", path, match_gen);
+
+    d = opendir(path);
     if (!d) {
-	perror(leave_prefix);
+	perror(path);
 	cleanup_and_exit(1);
     }
-    ncached_anims = 0;
     while (e = readdir(d)) {
 	anim_t *an = &cached_anims[ncached_anims];
 	int i, dummy;
 	if (match_gen) {
-	    int old_gen = -1;
 	    if (0 == strncmp("gen", e->d_name, 3)) {
 		sscanf(e->d_name, "gen%d", &old_gen);
 		checked_gen = 1;
@@ -368,6 +404,7 @@ update_cached_anims(int match_gen) {
 		}
 	    }
 	}
+	if (e->d_name[0] == '.') continue;
 	if (filename_is_xxx(e->d_name)) {
 	    if (4 != sscanf(e->d_name, "%d=%d=%d=%d.xxx",
 			    &an->generation, &an->id,
@@ -381,7 +418,7 @@ update_cached_anims(int match_gen) {
 		}
 	    }
 	    if (i == nserver_anims && nserver_anims) {
-		snprintf(fbuf, MAXBUF, "%s%s", leave_prefix, e->d_name);
+		snprintf(fbuf, MAXBUF, "%s%s", path, e->d_name);
 		if (debug)
 		    printf("removing marker %s nserver_anims=%d\n",
 			   fbuf, nserver_anims);
@@ -389,18 +426,27 @@ update_cached_anims(int match_gen) {
 		continue;
 	    }
 	    ncached_anims++;
+	    cached_anims = realloc(cached_anims, (1+ncached_anims)*sizeof(anim_t));
+	    continue;
+	}
+
+	snprintf(fbuf, PATH_MAX, "%s%s", path, e->d_name);
+	if (-1 == stat(fbuf, &sbuf)) continue;
+
+	if (0 == sbuf.st_size) continue;
+
+	if (S_ISDIR(sbuf.st_mode)) {
+	    strncat(fbuf, "/", PATH_MAX);
+	    update_cached_anims_path(fbuf, 0);
 	    continue;
 	}
  
 	if (!filename_is_mpg(e->d_name)) continue;
-	snprintf(fbuf, MAXBUF, "%s%s", leave_prefix, e->d_name);
 
 	if (4 != sscanf(e->d_name, "%d=%d=%d=%d.mpg",
 			&an->generation, &an->id, &an->first, &an->last)) {
 	    continue;
 	}
-	    
-	if (-1 == stat(fbuf, &sbuf)) continue;
 
 	if ((bracket_begin_id != -1 && an->id < bracket_begin_id) ||
 	    (bracket_end_id != -1 && an->id > bracket_end_id))
@@ -425,33 +471,15 @@ update_cached_anims(int match_gen) {
 	an->ctime = sbuf.st_ctime;
 	an->url[0] = 0;
 	an->type = 0;
+	strncpy(an->path, path, PATH_MAX);
 	ncached_anims++;
+	cached_anims = realloc(cached_anims, (1+ncached_anims)*sizeof(anim_t));
     }
     closedir(d);
-    if (debug > 1)
-	print_anims("ncached_anims", cached_anims, ncached_anims);
 
     if (match_gen && !checked_gen) write_gen = 1;
 
-    if (clear_gen) {
-	DIR *d = opendir(leave_prefix);
-	if (debug) printf("killing all sheep!\n");
-
-	if (!d) {
-	    perror(leave_prefix);
-	    cleanup_and_exit(1);
-	}
-
-	while (e = readdir(d)) {
-	    if ((0 == strncmp("gen", e->d_name, 3)) ||
-		filename_is_mpg(e->d_name) ||
-		filename_is_xxx(e->d_name)) {
-		snprintf(fbuf, MAXBUF, "%s%s", leave_prefix, e->d_name);
-		unlink(fbuf);
-	    }
-	}
-	closedir(d);
-    }
+    if (clear_gen) clear_path(path, old_gen);
 	
     if (write_gen) {
 	if (debug) printf("write gen %d.\n", generation);
@@ -460,6 +488,16 @@ update_cached_anims(int match_gen) {
     }
 }
 
+void update_cached_anims(int match_gen) {
+    ncached_anims = 0;
+    if (NULL == cached_anims)
+	cached_anims = malloc(sizeof(anim_t));
+    update_cached_anims_path(leave_prefix, match_gen);
+    if (debug > 1)
+	print_anims("ncached_anims", cached_anims, ncached_anims);
+}
+
+
 static int
 irandom(int n) {
     return random()%n;
@@ -468,13 +506,13 @@ irandom(int n) {
 void
 cached_file_name(char *buf, anim_t *an) {
     snprintf(buf, MAXBUF, "%s%05d=%05d=%05d=%05d.mpg",
-	    leave_prefix, an->generation, an->id, an->first, an->last);
+	    an->path, an->generation, an->id, an->first, an->last);
 }
 
 void
 deleted_file_name(char *buf, anim_t *an) {
     snprintf(buf, MAXBUF, "%s%05d=%05d=%05d=%05d.xxx",
-	    leave_prefix, an->generation, an->id, an->first, an->last);
+	    an->path, an->generation, an->id, an->first, an->last);
 }
 
 void
@@ -486,55 +524,12 @@ not_playing() {
     }
 }    
 
-
-
-#define max_plays 1000
-
 void
 print_stats() {
-    int i, n, j;
-
-    int buck[max_plays];
-
-    for (i = 0; i < max_plays; i++)
-	buck[i] = 0;
-
     printf("nplays = %d, ", nplays);
     printf("nloopplays = %d, ", nloopplays);
     printf("nrestarts = %d, ", nrestarts);
     printf("ncached_anims = %d\n", ncached_anims);
-     
-    printf("nplays_by_id:");
-    for (i = 0; i < max_anims; i++) {
-	if (nplays_by_id[i] != 0) {
-	    printf(" %d:%d", i, nplays_by_id[i]);
-	    if (nplays_by_id[i] < max_plays)
-		buck[nplays_by_id[i]]++;
-	}
-    }
-    printf("\n");
-    
-    printf("zeroes:");
-    for (i = 0; i < ncached_anims; i++) {
-	if (0 == nplays_by_id[cached_anims[i].id]) {
-	    printf(" %d", cached_anims[i].id);
-	    buck[0]++;
-	}
-    }
-    printf("\n");
-
-    n = 0;
-    for (i = 0; i < max_plays; i++) {
-	if (buck[i] > 0) n = i;
-    }
-    
-    printf("histogram:\n");
-    for (i = 0; i <= n; i++) {
-	printf("%4d %4d ", i, buck[i]);
-	for (j = 0; j < buck[i]; j++)
-	    putchar('o');
-	printf("\n");
-    }
 }
 
 /* traverse the graph of anims, and writes the mpeg files into stdin
@@ -604,7 +599,6 @@ do_display() {
 			  idx, current_anim_id, niters);
 	if (stats) print_stats();
 
-	nplays_by_id[current_anim_id]++;
 	nplays++;
 	if (cached_anims[idx].first == cached_anims[idx].last) {
 	    nloopplays++;
@@ -700,6 +694,7 @@ do_display() {
 	for (h = history_size; h >= 0; h--) {
 	    for (i = 0; i < ncached_anims; i++) {
 		if (!cached_anims[i].deleted &&
+		    (cached_anims[i].generation == cached_anims[idx].generation) &&
 		    cached_anims[i].first == sym) {
 		    int hh;
 		    for (hh = 0; hh < h; hh++) {
@@ -1161,8 +1156,10 @@ list_start_element(void *userData, const char *name, const char **atts) {
 	if (!strcmp(state, "done") && (0 == an->type)) {
 	    if (max_anims == nserver_anims+1) {
 		fprintf(stderr, "too many sheep.\n");
-	    } else
+	    } else {
 		nserver_anims++;
+		server_anims = realloc(server_anims, (1+nserver_anims) * sizeof(anim_t));
+	    }
 	} else if (!strcmp(state, "expunge")) {
 	    char buf[MAXBUF];
 	    if (debug) printf("expunging id=%d.\n", an->id);
@@ -1222,6 +1219,9 @@ void update_server_anims() {
     }
 
     nserver_anims = 0;
+    if (NULL == server_anims)
+	server_anims = malloc(sizeof(anim_t));
+
     generation = -1;
 
     do {
@@ -1682,12 +1682,12 @@ flags_init(int *argc, char ***argv) {
 	exit(1);
     }
 
-    if (leave_prefix && strlen(leave_prefix) > MAXPATHLEN) {
+    if (leave_prefix && strlen(leave_prefix) > PATH_MAX) {
 	fprintf(stderr, "save-dir too long: %s.\n", leave_prefix);
 	exit(1);
     }
 	
-    if (splash_prefix && strlen(splash_prefix) > MAXPATHLEN) {
+    if (splash_prefix && strlen(splash_prefix) > PATH_MAX) {
 	fprintf(stderr, "data-dir too long: %s.\n", splash_prefix);
 	exit(1);
     }
@@ -1750,7 +1750,7 @@ flags_init(int *argc, char ***argv) {
 	    fprintf(stderr, "HOME envar not defined\n");
 	    cleanup_and_exit(1);
 	}
-	if (strlen(hom) > MAXPATHLEN) {
+	if (strlen(hom) > PATH_MAX) {
 	    fprintf(stderr, "HOME envar too long: %s.\n", hom);
 	    cleanup_and_exit(1);
 	}
@@ -1872,12 +1872,6 @@ main(int argc, char **argv) {
     signal(SIGTERM, handle_sig_term);
     signal(SIGINT, handle_sig_term);
 
-    if (1) {
-	int i;
-	for (i = 0; i < max_anims; i++)
-	    nplays_by_id[i] = 0;
-    }
-
     flags_init(&argc, &argv);
 
     if (1) {
