@@ -18,7 +18,7 @@
 */
 
 static char *flam3_genome_c_id =
-"@(#) $Id: flam3-genome.c,v 1.10 2005/05/21 01:19:36 spotspot Exp $";
+"@(#) $Id: flam3-genome.c,v 1.11 2005/07/20 06:06:13 spotspot Exp $";
 
 #include "private.h"
 
@@ -210,6 +210,7 @@ main(argc, argv)
    double speed = argf("speed", 0.1);
    int verbose = argi("verbose", 0);
    int bits = argi("bits", 64);
+   int ntries = argi("tries", 50);
    int seed = argi("seed", 0);
    flam3_genome *parent0, *parent1;
    int parent0_n, parent1_n;
@@ -450,6 +451,82 @@ main(argc, argv)
 	   if (debug) fprintf(stderr, "adding symmetry\n");
 	   note("symmetry");
 	   flam3_add_symmetry(&cp_orig, 0);
+	 } else if (r < 0.6) {
+	   int b = 1 + random()%6;
+	   int same = random()&3;
+	   if (debug) fprintf(stderr, "setting post xform\n");
+	   note("post");
+	   for (i = 0; i < flam3_nxforms; i++) {
+	     int copy = (i > 0) && same;
+	     if (cp_orig.xform[i].density == 0.0) continue;
+	     if (copy) {
+	       for (j = 0; j < 3; j++) {
+		 cp_orig.xform[i].post[j][0] = cp_orig.xform[0].post[j][0];
+		 cp_orig.xform[i].post[j][1] = cp_orig.xform[0].post[j][1];
+	       }
+	     } else {
+	       if (b&1) {
+		 double f = M_PI * flam3_random11();
+		 double t[2][2];
+		 t[0][0] = (cp_orig.xform[i].c[0][0] * cos(f) +
+			    cp_orig.xform[i].c[0][1] * -sin(f));
+		 t[0][1] = (cp_orig.xform[i].c[0][0] * sin(f) +
+			    cp_orig.xform[i].c[0][1] * cos(f));
+		 t[1][0] = (cp_orig.xform[i].c[1][0] * cos(f) +
+			    cp_orig.xform[i].c[1][1] * -sin(f));
+		 t[1][1] = (cp_orig.xform[i].c[1][0] * sin(f) +
+			    cp_orig.xform[i].c[1][1] * cos(f));
+
+		 cp_orig.xform[i].c[0][0] = t[0][0];
+		 cp_orig.xform[i].c[0][1] = t[0][1];
+		 cp_orig.xform[i].c[1][0] = t[1][0];
+		 cp_orig.xform[i].c[1][1] = t[1][1];
+
+		 f *= -1.0;
+
+		 t[0][0] = (cp_orig.xform[i].post[0][0] * cos(f) +
+			    cp_orig.xform[i].post[0][1] * -sin(f));
+		 t[0][1] = (cp_orig.xform[i].post[0][0] * sin(f) +
+			    cp_orig.xform[i].post[0][1] * cos(f));
+		 t[1][0] = (cp_orig.xform[i].post[1][0] * cos(f) +
+			    cp_orig.xform[i].post[1][1] * -sin(f));
+		 t[1][1] = (cp_orig.xform[i].post[1][0] * sin(f) +
+			    cp_orig.xform[i].post[1][1] * cos(f));
+
+		 cp_orig.xform[i].post[0][0] = t[0][0];
+		 cp_orig.xform[i].post[0][1] = t[0][1];
+		 cp_orig.xform[i].post[1][0] = t[1][0];
+		 cp_orig.xform[i].post[1][1] = t[1][1];
+
+	       }
+	       if (b&2) {
+		 double f = 0.2 + flam3_random01();
+		 double g;
+		 if (random()&1) f = 1.0 / f;
+		 if (random()&1) {
+		   g = 0.2 + flam3_random01();
+		   if (random()&1) g = 1.0 / g;
+		 } else
+		   g = f;
+		 cp_orig.xform[i].c[0][0] /= f;
+		 cp_orig.xform[i].c[0][1] /= f;
+		 cp_orig.xform[i].c[1][1] /= g;
+		 cp_orig.xform[i].c[1][0] /= g;
+		 cp_orig.xform[i].post[0][0] *= f;
+		 cp_orig.xform[i].post[1][0] *= f;
+		 cp_orig.xform[i].post[0][1] *= g;
+		 cp_orig.xform[i].post[1][1] *= g;
+	       }
+	       if (b&4) {
+		 double f = flam3_random11();
+		 double g = flam3_random11();
+		 cp_orig.xform[i].c[2][0] -= f;
+		 cp_orig.xform[i].c[2][1] -= g;
+		 cp_orig.xform[i].post[2][0] += f;
+		 cp_orig.xform[i].post[2][1] += g;
+	       }
+	     }
+	   }
 	 } else if (r < 0.7) {
 	   // change just the color
 	   if (debug) fprintf(stderr, "mutating color\n");
@@ -618,9 +695,9 @@ main(argc, argv)
      } while ((avg_pix < avg_thresh ||
 	       fraction_black < black_thresh ||
 	       fraction_white > white_limit) &&
-	      count < 50);
+	      count < ntries);
 
-     if (50 == count) {
+     if (ntries == count) {
        fprintf(stderr, "warning: reached maximum attempts, giving up.\n");
      }
 
