@@ -18,7 +18,7 @@
 */
 
 static char *libifs_c_id =
-"@(#) $Id: flam3.c,v 1.8 2005/05/21 01:19:36 spotspot Exp $";
+"@(#) $Id: flam3.c,v 1.9 2005/07/20 06:06:16 spotspot Exp $";
 
 
 #include "private.h"
@@ -475,6 +475,11 @@ double flam3_dimension(flam3_genome *cp, int ntries, int clip_to_camera) {
 	got += 2;
       } else {
 	nclipped++;
+	if (nclipped > 10 * ntries) {
+	    fprintf(stderr, "warning: too much clipping, "
+		    "flam3_dimension giving up.\n");
+	    return sqrt(-1.0);
+	}
       }
       i4 += 4;
     }
@@ -1231,14 +1236,16 @@ int flam3_random_bit() {
 
 
 /* sum of entries of vector to 1 */
-static void normalize_vector(double *v, int n) {
+static int normalize_vector(double *v, int n) {
     double t = 0.0;
     int i;
     for (i = 0; i < n; i++)
 	t += v[i];
+    if (0.0 == t) return 1;
     t = 1.0 / t;
     for (i = 0; i < n; i++)
 	v[i] *= t;
+    return 0;
 }
 
 
