@@ -19,12 +19,12 @@
 
 
 static char *jpeg_h_id =
-"@(#) $Id: img.h,v 1.5 2005/05/07 01:45:34 spotspot Exp $";
+"@(#) $Id: img.h,v 1.6 2005/07/20 06:06:16 spotspot Exp $";
 
 #include <stdio.h>
 
-void write_jpeg(FILE *file, char *image, int width, int height);
-void write_png(FILE *file, char *image, int width, int height);
+void write_jpeg(FILE *file, unsigned char *image, int width, int height);
+void write_png(FILE *file, unsigned char *image, int width, int height);
 
 /* returns RGBA pixel array or NULL on failure */
 unsigned char *read_png(FILE *file, int *width, int *height);
