@@ -141,7 +141,8 @@ static void fifo_init()
     char *hom = getenv("HOME");
     char fname[MAXPATHLEN];
 
-    
+
+    /* assuming the cache dir is ~/.sheep is wrong xxx */
     sprintf(fname, "%s/.sheep/overlay_fifo", hom);
 
     unlink(fname);
@@ -161,7 +162,7 @@ static void fifo_init()
 
 static void overlay_read_png(FILE *ifp)
 {
-  char sig_buf [SIG_CHECK_SIZE];
+  unsigned char sig_buf [SIG_CHECK_SIZE];
   png_struct *png_ptr;
   png_info *info_ptr;
   png_byte **png_image;
@@ -302,8 +303,10 @@ static void overlay_read_ppm(FILE *fin)
     overlay_luma = malloc(overlay_width * overlay_height * 2);
     p = (short *) overlay_luma;
 #else
-    p = overlay_luma = malloc(overlay_width * overlay_height);
-    q = overlay_alpha = malloc(overlay_width * overlay_height);
+    overlay_luma = malloc(overlay_width * overlay_height);
+    p = (unsigned char *) overlay_luma;
+    overlay_alpha = malloc(overlay_width * overlay_height);
+    q = (unsigned char *) overlay_alpha;
 #endif
     for (i = 0; i < overlay_height; i++)
 	for (j = 0; j < overlay_width; j++) {
@@ -542,8 +545,12 @@ static int open_display (x11_instance_t * instance)
 		       (CWBackPixmap | CWBackingStore | CWBorderPixel |
 			CWEventMask | CWColormap), &attr);
     } else {
-      /* non-zoomed on the window specified on the command line */
-      instance->window = window_id;
+	/* zoomed to fit the window specified on the command line */
+	XWindowAttributes xgwa;
+	XGetWindowAttributes(instance->display, window_id, &xgwa);
+	instance->window = window_id;
+	instance->displaywidth = xgwa.width;
+	instance->displayheight = xgwa.height;
     }
 
     instance->gc = XCreateGC (instance->display, instance->window, 0,
@@ -632,7 +639,7 @@ static vo_frame_t * x11_get_frame (vo_instance_t * _instance, int flags)
     while (frame->wait_completion)
 	x11_event (instance);
 
-    frame->rgb_ptr = frame->ximage->data;
+    frame->rgb_ptr = (uint8_t *) frame->ximage->data;
     frame->rgb_stride = frame->ximage->bytes_per_line;
     frame->yuv_stride = instance->width;
     if ((flags & VO_TOP_FIELD) == 0)
@@ -663,7 +670,7 @@ static void x11_field (vo_frame_t * _frame, int flags)
     x11_frame_t * frame;
 
     frame = (x11_frame_t *) _frame;
-    frame->rgb_ptr = frame->ximage->data;
+    frame->rgb_ptr = (uint8_t *) frame->ximage->data;
     if ((flags & VO_TOP_FIELD) == 0)
 	frame->rgb_ptr += frame->ximage->bytes_per_line;
 }
@@ -719,7 +726,7 @@ static int x11_alloc_frames (x11_instance_t * instance)
 	    return 1;
 	}
 
-	instance->frame[i].ximage->data = alloc;
+	instance->frame[i].ximage->data = (char *) alloc;
 	alloc += size;
     }
 
@@ -870,7 +877,7 @@ static int xv_check_extension (x11_instance_t * instance)
 	return 1;
     }
 
-    XvQueryAdaptors (instance->display, instance->window, &adaptors,
+    XvQueryAdaptors (instance->display, instance->window, (unsigned int *) &adaptors,
 		     &adaptorInfo);
 
     for (i = 0; i < adaptors; i++)
@@ -913,7 +920,7 @@ static int xv_alloc_frames (x11_instance_t * instance)
 	instance->frame[i].wait_completion = 0;
 	instance->frame[i].xvimage =
 	    XvShmCreateImage (instance->display, instance->port, FOURCC_YV12,
-			      alloc, instance->width, instance->height,
+			      (char *) alloc, instance->width, instance->height,
 			      &(instance->shminfo));
 	if ((instance->frame[i].xvimage == NULL) ||
 	    (instance->frame[i].xvimage->data_size != 6 * size)) { /* FIXME */
